#ifndef _COMMON_H_
#define _COMMON_H_

#include "basec.h"
#include "confuse.h"
#include <string.h>
#include <sys/stat.h>
#include <SDL.h>
#include <SDL_ttf.h>
#include <SDL_image.h>
#include <SDL_mixer.h>
#include <png.h>

#define TMP_BOOT_FILE "/tmp/boot.ini"

#define COMMAND_THEMESELECT "!themeselect"
#define COMMAND_BACKGROUNDSELECT "!backgroundselect"
#define COMMAND_COLORSELECT "!colorselect"
#define COMMAND_SYSCONFIG "!sysconfig"
#define COMMAND_UNLOCK_MENU "!unlockmenu"
#define COMMAND_HANDLESELECT "!/usr/local/dmenu/pad/padon.sh"   //alterac add  for wirile
#define COMMAND_HANDLENOSELECT "!/usr/local/dmenu/pad/padoff.sh"

#define COMMAND_BACKLIGHT1SELECT "!/usr/local/dmenu/backlight_1.sh"
#define COMMAND_BACKLIGHT2SELECT "!/usr/local/dmenu/backlight_2.sh"
#define COMMAND_BACKLIGHT3SELECT "!/usr/local/dmenu/backlight_3.sh"
#define COMMAND_BACKLIGHT4SELECT "!/usr/local/dmenu/backlight_4.sh"
#define COMMAND_BACKLIGHT5SELECT "!/usr/local/dmenu/backlight_5.sh"

//backlight time
#define COMMAND_TIME0SELECT "!/usr/local/dmenu/DimmerDelay_0.sh"
#define COMMAND_TIME1SELECT "!/usr/local/dmenu/DimmerDelay_5.sh"
#define COMMAND_TIME2SELECT "!/usr/local/dmenu/DimmerDelay_10.sh"
#define COMMAND_TIME3SELECT "!/usr/local/dmenu/DimmerDelay_15.sh"
#define COMMAND_TIME4SELECT "!/usr/local/dmenu/DimmerDelay_20.sh"
#define COMMAND_TIME5SELECT "!/usr/local/dmenu/DimmerDelay_30.sh"
#define COMMAND_TIME6SELECT "!/usr/local/dmenu/DimmerDelay_60.sh"

//langage
#define COMMAND_CHSELECT "!/usr/local/dmenu/lan_chn.sh"
#define COMMAND_ENSELECT "!/usr/local/dmenu/lan_en.sh"
#define COMMAND_JAPSELECT "!/usr/local/dmenu/lan_jap.sh"
#define COMMAND_SPAINSELECT "!/usr/local/dmenu/lan_spain.sh"
#define COMMAND_RUSSELECT "!/usr/local/dmenu/lan_rus.sh"
//sound
#define COMMAND_SOUND0SELECT "!/usr/local/dmenu/resources/sounds/sound_1"
#define COMMAND_SOUND1SELECT "!/usr/local/dmenu/resources/sounds/sound_2"
#define COMMAND_SOUND2SELECT "!/usr/local/dmenu/resources/sounds/sound_3"
#define COMMAND_SOUND3SELECT "!/usr/local/dmenu/resources/sounds/sound_4"
#define COMMAND_SOUND4SELECT "!/usr/local/dmenu/resources/sounds/sound_5"

//tv
#define COMMAND_TVOPENSELECT "!/usr/local/dmenu/tv/tvon_pal.sh"
#define COMMAND_TVCLOSESELECT "!/usr/local/dmenu/tv/tvoff.sh"

//udc
#define COMMAND_UDCOPENSELECT "!/usr/local/dmenu/udc/open.sh"
#define COMMAND_UDCCLOSESELECT "!/usr/local/dmenu/udc/close.sh"
#define COMMAND_GSENSOROPEN "!/usr/local/dmenu/gsensor/gsensor_state_on.sh"
#define COMMAND_GSENSORCLOSE "!/usr/local/dmenu/gsensor/gsensor_state_off.sh"

#define COMMAND_A320_ZOOM_IN_MODE  "!/usr/local/dmenu/Set_a320_zoom_in.sh"
#define COMMAND_A320_ZOOM_OUT_MODE "!/usr/local/dmenu/Set_a320_zoom_out.sh"
#define COMMAND_A320_CLOSE_MODE  "!/usr/local/dmenu/Set_a320_close.sh"

#define free_sound(snd)    clean_erase(snd, Mix_FreeChunk)
#define free_surface(sfc)  clean_erase(sfc, SDL_FreeSurface)
#define free_font(fnt)     clean_erase(fnt, TTF_CloseFont)
#define free_erase(e)      clean_erase(e, free)
#define free_color(c)      free_erase(c)

#define internal_command(s) (s[0] == '!')

#define getKeyDir(k) ((k==DINGOO_BUTTON_VOL_UP || k==DINGOO_BUTTON_R || k==DINGOO_BUTTON_Y || k==DINGOO_BUTTON_LEFT || k==DINGOO_BUTTON_UP)?PREV:NEXT)

#define MAX_LAN 4
#define GAMELOCK_ENABLED 0

typedef enum { MAINMENU, FILELIST, IMAGEVIEWER, COLORPICKER } MenuState;
//typedef enum { THEMESELECT, BACKGROUNDSELECT, COLORSELECT } InternalCommand; 
typedef enum { THEMESELECT, BACKGROUNDSELECT, COLORSELECT,HANDLESELECT,HANDLENOSELECT,BACKLIGHT1SELECT,BACKLIGHT2SELECT,BACKLIGHT3SELECT,BACKLIGHT4SELECT,BACKLIGHT5SELECT,TVOPENSELECT,TVCLOSESELECT,UDCOPENSELECT,UDCCLOSESELECT,GSENSOROPEN,GSENSORCLOSE,SYSCONFIG,SOUND0SELECT,SOUND1SELECT,SOUND2SELECT,SOUND3SELECT,SOUND4SELECT,TIME0SELECT,TIME1SELECT,TIME2SELECT,TIME3SELECT,TIME4SELECT,TIME5SELECT,TIME6SELECT,CHSELECT,ENSELECT,JAPSELECT,SPAINSELECT,RUSSELECT,A320_ZOOM_IN_MODE,A320_CLOSE_MODE,A320_ZOOM_OUT_MODE,UNLOCK_MENU} InternalCommand; 
typedef enum { PREV, NEXT, UP, DOWN, RIGHT, LEFT } Direction;
typedef enum { SHUTDOWN, RELOAD_THEME, RELOAD_MENU } DeinitLevel;
typedef enum { NORMAL,USB,POWER} UDC_STATE;

//InternalCommand get_command(char* cmd);
int get_command(char* cmd);

typedef struct {
    char* file;
    struct stat* orig_stat;
    SDL_Surface *surface;
} ImageExportJob;


void run_command (char* cmd, char* args, char* workdir);
void run_internal_command (char* cmd, char* args, char* workdir);
void execute_next_command(char* dir, char** args);
void execute_inline_command(char* dir, char** args);
void clear_last_command();
char** build_arg_list(char* commandline, char* args); 
void free_arg_list(char** args);
int change_dir(char* path);
int is_back_dir(const struct dirent *dr);
int alphasort_i(const struct dirent** a, const struct dirent** b);
int filesystem_writeable();

FILE* load_file_and_handle_fail ( char* file, char* mode, int die_on_fail );
FILE* load_file( char* file, char* mode );
FILE* load_file_or_die( char* file, char* mode );
char* relative_file(char* path, char* file);
char* read_first_line( char* file);

SDL_Surface* load_image_file( char* file );
SDL_Surface* load_image_file_no_alpha( char* file );
SDL_Surface* load_image_file_with_format( char* file , int alpha, int fail_on_notfound );
SDL_Surface* load_resized_image(char* file, int width, int height);
void export_image(ImageExportJob* job);

SDL_Surface* render_text( char* text, TTF_Font* font, SDL_Color* color, int solid );
SDL_Surface* draw_text( char* text, TTF_Font* font, SDL_Color* color);
SDL_Color*   parse_color_string( char* color );
SDL_Color*   load_color_file( char* file );

void  init_rect(SDL_Rect* rect, int x, int y, int w, int h);
void  init_rect_pos(SDL_Rect* rect, int x, int y);
SDL_Surface* create_surface(int w, int h, int depth, int color, int a);
void alphaBlendSurface( SDL_Surface* s, int alpha );
void blitSurfaceAlpha ( SDL_Surface* src, SDL_Rect* src_rect, SDL_Surface* dst, SDL_Rect* dst_rect, int alpha);
SDL_Surface* shrink_surface(SDL_Surface *src, int width, int height);
int export_surface_as_bmp(char* file_name, SDL_Surface* surface);
SDL_Surface* tint_surface(SDL_Surface* src, int color, int alpha);
SDL_Surface* copy_surface(SDL_Surface* src);
#endif
