/*
 *  Copyright (C) 2009 sca <scahoo <at> gmail <dot> com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include "dosd/dosd.h"
#include "dosd/images.h"
#include "env.h"
#include "common.h"
#include "resource.h"

#define ADJUST_DELAY_TIME 600000
extern int image_pallette[3];
extern cfg_t *cfg_main;
bool show_speed;
SDL_Rect speed_dst_rect1, speed_dst_rect2;
TTF_Font* speed_font;
SDL_Color* dosd_color;
SDL_Surface* bat_change_face,*bat_full_face,*bat_null_face;
SDL_Surface* bat_full_big_face,*bat_null_big_face;

int bat_per = 0;
int adjust_time;

// ___ Globals ___________________________________________
// Global state
struct {
    FILE* proc_battery;
    FILE* proc_gpio3;
    FILE* proc_gpio1;
	FILE* proc_cgm;
	int mhz;
    bool is_locked;
    bool is_charging;
    BatteryState battery;
    unsigned int update_counter;
    // Measured in SDL_GetTick()s
    uint32_t next_update;
} g_state;

// Processed images
image_t g_images[IMG_MAX];

// ___ Functions ___________________________________________________
void _blit(SDL_Surface *surface, image_e which_image, ...);
void _update();
int battery_val;

// Color should be passed in RRGGBB format, e.g. 0xFFFFFF for white
int dosd_init()
{
    int i;

    memset(&g_state, 0, sizeof(g_state));
   // memset(&g_images, 0, sizeof(g_images));
    
#ifdef DINGOO_BUILD
    g_state.proc_battery = fopen(BATTERY_DEVICE, "rb");
    if (!g_state.proc_battery) goto init_error;
    
#if 0   //alterac  del
    g_state.proc_gpio1 = fopen(CHARGE_STATUS_DEVICE, "rb");
    if (!g_state.proc_gpio1) goto init_error;
    
    g_state.proc_gpio3 = fopen(LOCK_STATUS_DEVICE, "rb");
    if (!g_state.proc_gpio3) goto init_error;
#endif

#if 0
    g_state.proc_cgm = fopen(CPU_DEVICE, "r");
    if (!g_state.proc_cgm) goto init_error;
    show_speed = cfg_getbool(cfg_main, "SpeedDisp");
    //show_speed = false;

    init_rect_pos( &speed_dst_rect1, CPU_DISP_X, CPU_DISP_Y);
    init_rect_pos( &speed_dst_rect2, CPU_DISP_X+1, CPU_DISP_Y);
    speed_font = get_theme_font(CPU_FONT_SIZE);
#endif
    g_state.mhz = 0;
	g_state.is_charging = false;
	adjust_time = 0;
#endif
#if 1
	bat_change_face = load_image_file(RES_BAT_CHANGE);
	bat_null_face = load_image_file(RES_BAT_NULL);
	bat_full_face = load_image_file(RES_BAT_FULL);
	bat_full_big_face = load_image_file(RES_BIG_BAT_FULL);
	bat_null_big_face = load_image_file(RES_BIG_BAT_NULL);
#endif
    
    //dosd_color = get_theme_font_color();
    
#if 0
    for (i = 0; i < IMG_MAX; i++)
    {
        const image_data_t* image_data;
        image_t* image;
        uint32_t* buf;
        SDL_Surface* surface;
        int j;
        
        image_data =  c_images[i];
        image      = &g_images[i];
        
        image->w  = image_data->w;
        image->h  = image_data->h;
        
        buf = malloc(image->w * image->h * 4);
        if (!buf) goto init_error;
        
        for (j = 0; j < image->w * image->h; j++)
        {
            buf[j] = image_pallette[image_data->data[j]];
        }
        
		int color = (dosd_color->r) << 24;
		color += (dosd_color->g) << 16;
		color += (dosd_color->b) << 8;

		surface = SDL_CreateRGBSurfaceFrom(buf, image->w, image->h, 32, image->w * 4,
			(0xff<<24) & color,
			(0xff<<16) & color,
			(0xff<<8) & color,
			0xff);
        
        if (!surface) goto init_error;
        
        // Convert surface to target surface format
        image->sfc = SDL_DisplayFormatAlpha(surface);
        if (!image->sfc) 
        {
            SDL_FreeSurface(surface);
            goto init_error;
        }
        
        SDL_FreeSurface(surface);
        free(buf);
    }
#endif
    // Init g_state
    _update();
    
    return 1;
init_error:
    dosd_deinit();
    return 0;
}

void dosd_color_reset() {
//	dosd_color = get_theme_font_color();
}

void dosd_deinit()
{
    log_debug("De-initializing");
    int i;
    
#ifdef DINGOO_BUILD
	g_state.mhz=0;

    if (g_state.proc_battery)
        fclose(g_state.proc_battery);
    
#if 1
    if (g_state.proc_gpio1)
        fclose(g_state.proc_gpio1);
    
    if (g_state.proc_gpio3)
        fclose(g_state.proc_gpio3);

	if (g_state.proc_cgm)
		fclose(g_state.proc_cgm);
#endif
#endif
    SDL_FreeSurface(bat_change_face);
	SDL_FreeSurface(bat_null_face);
	SDL_FreeSurface(bat_full_face);
	SDL_FreeSurface(bat_null_big_face);
	SDL_FreeSurface(bat_full_big_face);

#if 0
    for (i = 0; i < IMG_MAX; i++)
    {
        if (g_images[i].sfc)
            SDL_FreeSurface(g_images[i].sfc);
    }
#endif
}
void sdl_draw_pic(int x,int y,SDL_Surface* src,SDL_Surface* dst)
{
		SDL_Rect icon_rect;
		init_rect_pos(&icon_rect,x,y);
		blitSurface(src,NULL,dst,&icon_rect);
}

void sdl_draw_subpic(SDL_Surface* src,SDL_Rect* src_rect,SDL_Surface* dst,SDL_Rect* dst_rect)
{
		blitSurface(src,src_rect,dst,dst_rect);
}

void draw_bat_state(SDL_Surface* screen)
{
		SDL_Rect rect_dst,rect_old;
		int divid_x;
		if (g_state.is_charging == 1 && g_state.battery != BAT_STATE_FULL)
			sdl_draw_pic(BATTERY_X,BATTERY_Y,bat_change_face,screen);
		else{
				int x = BATTERY_X;
				int y = BATTERY_Y;
				int full_x = BATTERY_X;
				int empty_x = BATTERY_X + BATTERY_W;
#if 1
				if (g_state.is_charging != 1){
					if (battery_val >= (BAT_LEVEL_BEST - 100)){
						g_state.battery = BAT_STATE_FULL;
					}
				}
#endif


				if (g_state.battery == BAT_STATE_FULL)
					divid_x = full_x;
				else if (g_state.battery == BAT_STATE_LEVEL1)
					divid_x = full_x + 12;
				else if (g_state.battery == BAT_STATE_LEVEL2)
					divid_x = full_x + 16;
				else if (g_state.battery == BAT_STATE_LEVEL3)
					divid_x = full_x + 20;
				else if (g_state.battery == BAT_STATE_EMPTY)
					divid_x = empty_x;
				else;
				init_rect(&rect_old,0,0,divid_x - x,BATTERY_H);
				init_rect_pos(&rect_dst,x,y);
				sdl_draw_subpic(bat_null_face,&rect_old,screen,&rect_dst);
				init_rect(&rect_old,divid_x - x,0,BATTERY_W - (divid_x - x),BATTERY_H);
				init_rect_pos(&rect_dst,divid_x,y);
				sdl_draw_subpic(bat_full_face,&rect_old,screen,&rect_dst);
		}
}
int loop = 0;
void draw_bat_cartoon(SDL_Surface * screen)
{
	SDL_Rect rect_dst,rect_old;
	int divid_w;
	if (adjust_time < 1800){
		g_state.battery = BAT_STATE_LEVEL1;
	}
	if (loop > 4){
		loop = 0;
	}
	if (g_state.is_charging == 1){
		if (g_state.battery == BAT_STATE_FULL){
			sdl_draw_pic(BATTERY_CHANGE_X,BATTERY_CHANGE_Y,bat_full_big_face,screen);
		}else{
			switch (loop){
				case 0 :  divid_w = BATTERY_CHANGE_W; break;
				case 1 :  divid_w = BATTERY_CHANGE_W + 50; break;
				case 2 :  divid_w = BATTERY_CHANGE_W + 100; break;
				case 3 :  divid_w = BATTERY_CHANGE_W + 150; break;
				case 4 :  divid_w = BATTERY_CHANGE_W + 200; break;
			}
			init_rect(&rect_old,0,0,divid_w,BATTERY_CHANGE_H);
			init_rect_pos(&rect_dst,0,0);
			sdl_draw_pic(BATTERY_CHANGE_X,BATTERY_CHANGE_Y,bat_null_big_face,screen);
			sdl_draw_subpic(bat_full_big_face,&rect_old,screen,&rect_dst);
		}
	}
	loop ++;

}
void dosd_show(SDL_Surface* surface)
{
	image_e battery_status;

	if (SDL_GetTicks() >= g_state.next_update)
    {
        _update();
    }
    
    // Battery
#if 0
    _blit(surface, IMG_BATTERY, -1);
    
    // This is a bit of a hack, as it relies on battery_state_e
    // To have the same index as image_e
    battery_status = g_state.battery;
	if ( g_state.is_charging ) battery_status = BAT_STATE_CHARGING;
    if (battery_status != BAT_STATE_EMPTY)
    {
        _blit(surface, battery_status, -1);
    }
#endif
	//alterac add
	draw_bat_state(surface);
	
    
    // Lock
#if 0
    if (g_state.is_locked)
    {
        _blit(surface, IMG_LOCK, IMG_BATTERY, -1);
    }

	if (show_speed) {
		SDL_Surface* speed_display;
		char speed_text[10] = " ";
		int disp_mhz = g_state.mhz;
		sprintf(speed_text, "%03d MHz", disp_mhz);
		speed_display = draw_text(speed_text, speed_font, dosd_color);
	    SDL_BlitSurface(speed_display, 0, surface, &speed_dst_rect1 );
	    SDL_BlitSurface(speed_display, 0, surface, &speed_dst_rect2 );
		free_surface(speed_display);
	}
#endif
}

inline bool dosd_is_locked() {
    return g_state.is_locked;
}

void _blit(SDL_Surface *surface, image_e which_image, ...) {
    /*
        image_e are expected in the variable arguments.
        The image will be offset by the combined width + padding
        of the images in the varargs.
        ALWAYS pass -1 as the last argument!
    */
    SDL_Rect dst;
    va_list ap;
    image_e  padding;
    image_t* image;
    
    if (which_image < 0 || which_image >= IMG_MAX)
        return;
    
    image = &g_images[which_image];
    
    dst.y = DOSD_PADDING;
    dst.x = surface->w - image->w - DOSD_PADDING; 
    
    va_start(ap, which_image);
    while ((padding = va_arg(ap, image_e)) != -1)
    {
        dst.x -= DOSD_PADDING + g_images[padding].w;
    }
    
    va_end(ap);
    
    SDL_BlitSurface(image->sfc, NULL, surface, &dst);
}

bool get_usb_connect_state()
{
	return g_state.is_charging;
}

void _update()   //alterac
{
#ifdef DINGOO_BUILD
    char buf[128];
    int mvolts;
    uint32_t gpio;
    
    rewind(g_state.proc_battery);
    fgets(buf, 127, g_state.proc_battery);
    sscanf(buf, "%d", &mvolts);

#if 0
    // Charge status
    rewind(g_state.proc_gpio1);
    fgets(buf, 127, g_state.proc_gpio1);
    sscanf(buf, "%x", &gpio);
    
    // FIXME: the GPIO bit indicating charge status seems
    // to randomly jump when a USB cable is attached.
	g_state.is_charging = ((gpio & GPIO_POWER_MASK) == 0);

    // Lock status
    rewind(g_state.proc_gpio3);
    fgets(buf, 127, g_state.proc_gpio3);
    sscanf(buf, "%x", &gpio);
    
    g_state.is_locked = ((gpio & GPIO_LOCK_MASK) == 0);
#endif

    g_state.is_charging = (mvolts&0x80000000);
    g_state.is_locked = false;//(mvolts&0x40000000);
    // Battery charge level
    mvolts = (mvolts&0x0000FFFF) * 4 + 183;    //alterac  adjust battery value
    battery_val = mvolts;
//    printf("mvolts  =   %d  charing = %d  locked = %d\n",mvolts,g_state.is_charging,g_state.is_locked);
    if (g_state.is_charging == 1){
	    if (mvolts >= BAT_LEVEL_BEST)	{
		    adjust_time ++;
	    }else{
		    adjust_time = 0;
	    }
    }
    if (mvolts >= BAT_LEVEL_BEST ) {  
	    g_state.battery = BAT_STATE_FULL;
    }
    else if (mvolts >= BAT_LEVEL_FAIR) g_state.battery = BAT_STATE_LEVEL1;
    else if (mvolts >= BAT_LEVEL_FAIR1) g_state.battery = BAT_STATE_LEVEL2;
    else if (mvolts >= BAT_LEVEL_LOW ) g_state.battery = BAT_STATE_LEVEL3;
    else                                  g_state.battery = BAT_STATE_EMPTY;


    // get mhz
#if 0
    if ( show_speed ) {
	    rewind(g_state.proc_cgm);
	    char line[80];
	    while( fgets(line, 80, g_state.proc_cgm) != NULL) {
		    if ( strstr(line, "PLL Freq") ) {
			    g_state.mhz = atoi(strstr(strstr(line, ": "), " "));
		    }
	    }
    }
#endif
#else
    g_state.is_locked   = false;
    g_state.is_charging = false;
    g_state.battery     = BAT_STATE_EMPTY;
	g_state.mhz			= 0;
#endif
    
    // Next update
    g_state.next_update = SDL_GetTicks() + DOSD_UPDATE_INTERVAL;
    g_state.update_counter++;
}

bool get_low_flag()
{
	if (g_state.battery == BAT_STATE_EMPTY && !g_state.is_charging)
		return 1;
	else
		return 0;
}
bool dosd_is_charging() {
	return g_state.is_charging;
}
