/*
 *  Copyright (C) 2009 Rookie1 <mr.rookie1@gmail.com>
 *                     Timothy Soehnlin <timothy.soehnlin@gmail.com>
 *
 *  Author: <mr.rookie1@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include <unistd.h>
#include <stdlib.h>
#include <signal.h>
#include <SDL.h>
#include <SDL_image.h>
#include "env.h" 
#include "dingoo.h"
#include "common.h"
#include "resource.h"
#include <stdio.h>

#include "main.h"
#include "menu.h"
#include "filelist.h"
#include "colorpicker.h"
#include "imageviewer.h"

#include "loading.h"
#include "conf.h"
#include "brightness.h"
#include "volume.h"
#include "watch.h"
#include "sound.h"
#include "persistent.h"
#include "dosd/dosd.h"
#include "jz4750_lcd.h"
#include "jz4750_lcd_def.h"
#include <sys/ioctl.h>

#define UDC_CONNECT_EXE "/usr/local/apps/udc/udc_connect.bin"
extern cfg_t *cfg_main;
extern cfg_t *cfg_lock;
extern int conf_changed;

static Uint32 next_time;
SDL_Surface *screen, *screen_cache, *screen_cache1;
extern SDL_Surface ** circle;
extern SDL_Surface *handle_face;
extern SDL_Surface * udc_face;
extern SDL_Surface * udc_fail_face;
extern SDL_Surface* background;
extern int a320_mode_flag;
static int udc_open_flag;
static int change_flag;
char change_state = 0;
static char change_old_state;
MenuState state = MAINMENU;
int program_done;
bool escape_held;
int timer = 0;
int bat_cartoon_flag ;
//FILE*fp_printf;
extern int isDrawFlag;
int udc_draw_flag;
extern  char handle_value,gsensor_open_flag;
extern int unlock_to_game;

Uint32 time_left(void)
{
    Uint32 now = SDL_GetTicks();
    return next_time <= now ? 0 : next_time - now;
}

void gsensor_init()
{
	gsensor_open_flag = (char)cfg_getint(cfg_main, "Gsensor_state");
	system("/usr/local/dmenu/gsensor/close.sh");
	return;
}

void a320_init()
{
	a320_mode_flag = (int)cfg_getint(cfg_main, "A320_state");
}

void game_unlock_init()
{
#if GAMELOCK_ENABLED
	unlock_to_game = (int)cfg_getint(cfg_lock,"Game_unlock");
	printf("%s %d unlock_to_game = %d\n", __func__, __LINE__, unlock_to_game);
	if(unlock_to_game > 1 || unlock_to_game < 0)
		unlock_to_game = 0;
#else
	unlock_to_game = 1;
#endif
}

int init_system() {

		//Move to dmenu root dir
		change_dir(DMENU_PATH);

		// load config
		if (conf_load(NULL)) {
				printf("conf load fail!\n");
				return 1;
		}
    
	    if (can_write_fs()) clear_last_command();
    
    // Read saved persistent state
    if (!persistent_init())
    {
        printf("Unable to initialize persistent memory");
    }
    
    //Init OSD 
    {
        brightness_init();
        volume_init();
		watch_init();
        if (!dosd_init())
        {
            printf("Unable to initialize OSD");
            return 1;
        }
    }
    //gsenser init
	gsensor_init();

    // Init sound
    sound_init();

//	enal_init();
    // init menu
    if (menu_init())
    {
        printf("Unable to load menu");
        return 1;
    }
    
    return 0;
}

int init_display() {
	int i;
	// initialize SDL video
	if ( SDL_Init( SDL_INIT_VIDEO) < 0 )
	{
		printf( "Unable to init SDL: %s", SDL_GetError() );
		return 1;
	}

	// make sure SDL cleans up before exit
	//atexit(SDL_Quit);   //alterac  del

	// disable mouse pointer
	SDL_ShowCursor(SDL_DISABLE);

	// create a new window
//	if (get_tv_flag() == 1){
//		screen = SDL_SetVideoMode(SCREEN_WIDTH+240, SCREEN_HEIGHT*2, SCREEN_COLOR_DEPTH, SDL_SWSURFACE);
//	}else{
		screen = SDL_SetVideoMode(SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_COLOR_DEPTH, SDL_SWSURFACE);
//	}
	if ( !screen )
	{
		printf("Unable to set %dx%d video: %s", SCREEN_WIDTH, SCREEN_HEIGHT, SDL_GetError());
		return 1;
	} else {
	}

	screen_cache = create_surface(SCREEN_WIDTH, SCREEN_HEIGHT, 24, 0xFFFFFF,0);
	if (screen_cache == NULL)
		printf("Unable to create screen cache\n");
//	screen_cache1 = create_surface(SCREEN_WIDTH, SCREEN_HEIGHT, 24, 0xFFFFFF,0);
//	if (screen_cache1 == NULL)
//		printf("Unable to create screen cache\n");

	//Ready fonts
	if ((i = TTF_Init()) != 0)
		printf("TTF init fail!\n");
	printf("screen init sccuss!\n");
	return 0;
}

int get_boot_flag()
{
	int value;
	FILE* fp = fopen(TMP_BOOT_FILE,"rb");
	if (fp == NULL){
		return 1;
	}
	fread(&value,sizeof(int),1,fp);
	//printf("........boot flag  is....... %d\n",value);
	fclose(fp);
	return 0;
}

int init() {
   // log_debug("Initializing");
   	//clear_a320_mode();
	open_sdl_key_even();
	dmenu_set_vol(0);
	set_cpu_freq(MAX_CPU_FREQ);

	int i = get_tvdev_value();
	printf("tvdev value = %d\n",i);
	if (i != 0){
		if (i == 2){
			system("/usr/local/dmenu/tv/tvon_pal.sh");
		}
		set_tv_flag(1);
	}else{
		set_tv_flag(0);
	}

	system("/usr/local/dmenu/tv/tvon_pal.sh");
//	bat_cartoon_flag = get_boot_flag();

	set_write_fs(filesystem_writeable()); 

#ifdef DINGOO_BUILD
	// Need to ignore SIGHUP if we are started by init.
	// In FB_CloseKeyboard() of SDL_fbevents.c , it does
	// ioctl(tty0_fd, TIOCNOTTY, 0) to detach the process from tty,
	// this will send a SIGHUP to us and dmenu will quit at 
	// SDL_Init() below if it's not ignored.

	if (signal(SIGHUP, SIG_IGN) == SIG_ERR) {
		log_error("Unable to ignore SIGHUP");
	}
#endif

	int rc =  init_display() || init_system();
	next_time = SDL_GetTicks() + TICK_INTERVAL;
	//state = MAINMENU;
	return rc;
}

extern unsigned char sdl_to_fb_flag;
extern int isOpenScreen;
void write_tmp_boot_flag()
{
	int value = 10;
	FILE* fp = fopen(TMP_BOOT_FILE,"w");
	if (fp == NULL){
		printf("can't open %s !\n",TMP_BOOT_FILE);
	}
	fwrite(&value,sizeof(int),1,fp);
	fflush(fp);
	fsync(fp);
	fclose(fp);
}
void deinit(DeinitLevel level) {
	log_debug("De-initializing, level %d", level);
	//alterac  add
	if (sdl_to_fb_flag){
		//loading_init_video(screen);
		//save_sdlface(screen);
	}
	else{
		//loading_init_other(screen);
		//save_sdlface(screen);
	}
	//end
	
	//alterac add ,write flag file to tmp dir
//	write_tmp_boot_flag();
	//end


	//save_vol_to_file();
	int shutdown = level == SHUTDOWN;

	// de-init everything
	colorpicker_deinit();
	filelist_deinit();
	imageviewer_deinit();
	state = MAINMENU;

	log_debug("shutdown = %d\n",shutdown);
	if (shutdown) 
	{   
		if (!isOpenScreen){
			stop_flush_screen();
		}
		sound_deinit();
		brightness_deinit();
		volume_deinit();
		dosd_deinit();
	}

	menu_deinit();

	if (level == SHUTDOWN) 
	{
		//Close down fonts
		TTF_Quit();
		conf_unload();
		loading_deinit();
	}
}

void reload(DeinitLevel level) {
	deinit(level);
	if (level == RELOAD_THEME) 
	{
		conf_reload_theme(THEME_NAME);
	}
	menu_init();
}

void draw_osd(SDL_Surface* screen) {
	dosd_show(screen);    
	if (volume_enabled())     volume_show(screen);
	//alterac del
	//if (brightness_enabled()) brightness_show(screen);
	if (watch_enabled())      watch_show(screen);
}


//alterac add


void draw_cartoon(SDL_Surface * face,SDL_Surface* face_all,int x,int y)
{
	int i;
	if (face == NULL){
		printf("this is no circle face!  error!\n");
		return;
	}
	Apply_surface(x,y,face,face_all,NULL);
}



void draw_udc_waittext(SDL_Surface * face)
{
	draw_cartoon(background,screen,0,0);
	draw_cartoon(face,screen,(SCREEN_WIDTH - face->w)/2,110);
	SDL_Flip(screen);
}

#if 0
void draw_sys_note()
{
	sdlDrawPicture((SCREEN_W - 200)/2,(SCREEN_H - 80)/2,PIC_NOTE,1);
	sdlDrawText_alignment(0,(SCREEN_H - 20)/2,str_sys_note[sys_language],textcolor);
	sdlFlipScreen();
}
#endif
//end

/**
 * Draw Screen
 */
int t = 0;
int superdraw_flag = 0;
void update_display() {

	// DRAWING STARTS HERE
	int recache = 0;
#if 0
	if (get_tv_flag() == 1){
		SDL_Surface* tmp_face = NULL;
		switch (state) {
			case MAINMENU:    recache = menu_draw(screen_cache1); break;
			case FILELIST:    recache = filelist_draw(screen_cache1); break;
			case IMAGEVIEWER: recache = imageviewer_draw(screen_cache1); break;  //background select  alterac
			case COLORPICKER: recache = colorpicker_draw(screen_cache1); break;  //color select
		}

		if (!recache) {
			SDL_BlitSurface(screen_cache, NULL, screen_cache1, NULL);
		} else {
			SDL_BlitSurface(screen_cache1, NULL, screen_cache, NULL);
		}

		switch (state) {
			case MAINMENU:    menu_osd(screen_cache1); draw_osd(screen_cache1);break;
			case FILELIST:    filelist_osd(screen_cache1); break;
			case IMAGEVIEWER: imageviewer_osd(screen_cache1); break;
			case COLORPICKER: colorpicker_osd(screen_cache1); break;
		}
		if (udc_open_flag){
			if (udc_draw_flag == 1){
				udc_draw_flag = 0;
				draw_cartoon(udc_face,screen_cache1,0,0);
				SDL_Flip(screen);
				SDL_Delay(50);
				stop_flush_screen();
			}
		}else if (change_flag){
			if (SDL_GetTicks() > timer){
				draw_bat_cartoon(screen_cache1);
				SDL_Flip(screen);
				timer = SDL_GetTicks() + DRAW_BAT_DELAY;
			}
		}
		else{
			if (SDL_GetTicks() > timer || ((superdraw_flag > 0)&&(superdraw_flag <= 10))){
				if (superdraw_flag > 0){
					superdraw_flag ++;

				}
				if (superdraw_flag >= 10)
					superdraw_flag = 0;
				if (t > 9){
					t = 0;
				}
				if (state == 0){
					draw_cartoon(circle[t],screen_cache1,67,134);
					if (handle_value == 1)
						draw_cartoon(handle_face,screen_cache1,HANDLE_X,OSD_Y);
					if (gsensor_open_flag)
						gsensor_show(screen_cache1);
				}
				if (get_sysabout_flag()){
					draw_sysabout(screen_cache1);
				}
				timer = SDL_GetTicks() + DRAW_DELAY;
				t++;
				if (!isDrawFlag){
					double scale_x = 1.0,scale_y = 1.0;
					scale_x = (double)(SCREEN_WIDTH+240)/(double)(SCREEN_WIDTH);
					scale_y = (double)(SCREEN_HEIGHT*2)/(double)(SCREEN_HEIGHT);
					tmp_face = zoomSurface(screen_cache1,scale_x,scale_y,1);
					SDL_BlitSurface(tmp_face, NULL, screen, NULL);
					SDL_Flip(screen);
					SDL_FreeSurface(tmp_face);
				}
				//SDL_SaveBMP(screen,"/usr/local/dmenu/test.bmp");
				//SDL_Delay(2000);
				//while(1);
			}	
		}
	}
	else{
#endif
		switch (state) {
			case MAINMENU:    recache = menu_draw(screen); break;
			case FILELIST:    recache = filelist_draw(screen); break;
			case IMAGEVIEWER: recache = imageviewer_draw(screen); break;  //background select  alterac
			case COLORPICKER: recache = colorpicker_draw(screen); break;  //color select
		}

		if (!recache) {
			SDL_BlitSurface(screen_cache, NULL, screen, NULL);
		} else {
			SDL_BlitSurface(screen, NULL, screen_cache, NULL);
		}

		switch (state) {
			case MAINMENU:    menu_animate(screen); break;
			case FILELIST:    filelist_animate(screen); break;
			case IMAGEVIEWER: imageviewer_animate(screen); break;
			case COLORPICKER: colorpicker_animate(screen); break;
		}

		switch (state) {
			case MAINMENU:    menu_osd(screen); draw_osd(screen);break;
			case FILELIST:    filelist_osd(screen); break;
			case IMAGEVIEWER: imageviewer_osd(screen); break;
			case COLORPICKER: colorpicker_osd(screen); break;
		}

		if (udc_open_flag){
			if (udc_draw_flag == 1 && !isDrawFlag){
				udc_draw_flag = 0;
				draw_cartoon(udc_face,screen,0,0);
				SDL_Flip(screen);
				SDL_Delay(100);
				stop_flush_screen();
				udc_loop();
				program_done = 1;
//				run_command(UDC_CONNECT_EXE, NULL,NULL);
			}
		}else if (change_flag){
			if (SDL_GetTicks() > timer){
				draw_bat_cartoon(screen);
				SDL_Flip(screen);
				timer = SDL_GetTicks() + DRAW_BAT_DELAY;
			}
		}
		else{
			if (SDL_GetTicks() > timer || ((superdraw_flag > 0)&&(superdraw_flag <= 10))){
				if (superdraw_flag > 0){
					superdraw_flag ++;

				}
				if (superdraw_flag >= 10)
					superdraw_flag = 0;
				if (t > 9){
					t = 0;
				}
				if (state == 0){
					draw_cartoon(circle[t],screen,80,(SCREEN_HEIGHT - 80 - MENU_TEXT_HEIGHT )/3 + 80 + MENU_TEXT_HEIGHT);
					if (handle_value == 1)
						draw_cartoon(handle_face,screen,HANDLE_X,OSD_Y);
					if (gsensor_open_flag)
						gsensor_show(screen);
				}
				if (get_sysabout_flag()){
					draw_sysabout(screen);
				}
				if(get_unlock_flag()){
					draw_unlock_view(screen);
				}
				timer = SDL_GetTicks() + DRAW_DELAY;
				t++;
				if (!isDrawFlag)
					SDL_Flip(screen);
			}	
		}

	// wait for remaining time of the frame
	SDL_Delay(time_left());
	next_time += TICK_INTERVAL;    
}

void handle_global_key(SDLKey key) {
	//Handle OSD activity
	Direction dir  = getKeyDir(key);
	if (state == MAINMENU) {
		if(escape_held) {
			log_debug("No functions defined yet for esc + key");
		} else {
			switch (key) {
#if 0
				case DINGOO_BUTTON_L:
				case DINGOO_BUTTON_R:
#else
				case DINGOO_BUTTON_VOL_UP:
				case DINGOO_BUTTON_VOL_DOWN:
#endif
					volume_change(dir);
					sound_out( GLOBAL_KEY );
					break;
#if 0
				case DINGOO_BUTTON_X:
				case DINGOO_BUTTON_Y:
					brightness_change(dir);
					sound_out( GLOBAL_KEY );
					break;
#endif
				default: break;
			}
		}
	}
}
void quit()
{
	program_done = 1;
}
int key_delay =0 ;
void set_timer()
{
	key_delay = 1;
}

void init_udc_state()
{
	udc_open_flag = 0;
	udc_draw_flag = 0;
	change_flag = 0;
	change_old_state = 0;
}
void listen() {

	// program main loop
	//int draw_is = 0;
	isDrawFlag = 0;
	int sound_time = 0,udc_check_time = 0;
	program_done = 0;
	key_delay = 0;
	SDLKey key;
	SDL_Event event;
	MenuState prevstate;
	escape_held = false;

	int last_key_time = SDL_GetTicks();
	int inactive_delay = cfg_getint(cfg_main, "DimmerDelay")*1000;
	int shutdown_delay = cfg_getint(cfg_main, "ShutdownDelay")*1000;
	int t = 0,j = 0,k = 0;

	timer = SDL_GetTicks() + DRAW_DELAY;  //alterac add
	sound_time = SDL_GetTicks() + SOUND_CLOSE_DELAY;
	udc_check_time = SDL_GetTicks() + UDC_DELAY_TIME;

	//Allow for easier menu nav
	SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY,SDL_DEFAULT_REPEAT_INTERVAL*3);

	//init udc state,so important dont't change value
	init_udc_state();
	gsensor_init();
	a320_init();
//	game_unlock_init();
	while (!program_done)
	{
		//if udc open,don't ack sdl evens
#if 0
		if (udc_open_flag){
			if (get_udc_state() != 1){
				udc_open_flag = 0;
				udc_draw_flag = 0;
				start_flush_screen();
				system("/usr/bin/udc_disconnect.sh");
				last_key_time = SDL_GetTicks() ;
				printf("\n udc device disconnected!\n");
			}
		}
#endif

		//check change state
		if (change_flag){
			if (get_udc_state() != 2){
				change_flag = 0;
				set_change_state(0);
				brightness_set_new();
				last_key_time = SDL_GetTicks() ;
			}
		}

		// message processing loop
		//while (SDL_PollEvent(&event) && !udc_open_flag)
		while (SDL_PollEvent(&event))
		{            

			// check for messages
			switch (event.type)
			{
				// exit if the window is closed
				case SDL_QUIT: 	quit(); break;

				case SDL_KEYUP:
						key = event.key.keysym.sym;
						//if (key == DINGOO_BUTTON_L || key == DINGOO_BUTTON_R){
						if (key == DINGOO_BUTTON_VOL_UP || key == DINGOO_BUTTON_VOL_DOWN){
							save_vol_to_file();
						}
						if (key == SDLK_ESCAPE) escape_held = false;
						break;

						// check for keypresses
				case SDL_KEYDOWN:
						//up freq
						//draw_is = 1;
						set_cpu_freq(MAX_CPU_FREQ);
						//alterac add  for gongfang
						sound_time = SDL_GetTicks() + SOUND_CLOSE_DELAY;
						if (!get_mute_flag())
							dmenu_set_vol(get_cur_vol());
						superdraw_flag = 1;
						key = event.key.keysym.sym;

						//if (dosd_is_locked()) break;

						last_key_time = SDL_GetTicks();

						if (brightness_is_dimmed()) {
							log_debug("no  ack  key evens!\n");
							brightness_dim(0);
							break;
						}
						//change state,alterac add
						if (change_flag == 1){
							//write_tmp_boot_flag();
							change_flag = 0;
							set_change_state(0);
							brightness_set_new();
							break;
						}
						//end

						prevstate = state;
						switch (state) {
							case MAINMENU:    state = menu_keypress(key); break;
							case FILELIST:    state = filelist_keypress(key); break;
							case IMAGEVIEWER: state = imageviewer_keypress(key); break;
							case COLORPICKER: state = colorpicker_keypress(key); break;
						}
						if (state == MAINMENU && prevstate != state) {
							menu_force_redraw(screen_cache);
						}

						brightness_dim(0);
						handle_global_key(key);
						break;
			} // end switch
		} // end of message processing

		if (SDL_GetTicks() > sound_time){
			if (!isDrawFlag)
				dmenu_set_vol(0);
			//redeced cpu freq
			//if (!udc_open_flag && !isDrawFlag){
			if (!isDrawFlag){
				if (get_tv_flag() == 0)
					set_cpu_freq(MIN_CPU_FREQ);
			}
			sound_time = SDL_GetTicks() + SOUND_CLOSE_DELAY;
		}

		//if ((inactive_delay > 0 && inactive_delay < (SDL_GetTicks()-last_key_time)) && !(brightness_is_dimmed()) && !udc_open_flag && !change_flag) {
		if ((inactive_delay > 0 && inactive_delay < (SDL_GetTicks()-last_key_time)) && !(brightness_is_dimmed())  && !change_flag) {
			if (!key_delay){
				printf("........dmenu set brightness_dim!...........\n");
				brightness_dim(1);
				//draw_is = 0;
				//system("echo 1 > /proc/sys/pm/suspend");
			}else{
				key_delay = 0;
				last_key_time = SDL_GetTicks();
			}	
		}

		//check udc 
		if (SDL_GetTicks() > udc_check_time){
			if (udc_open_flag == 0){
				if (get_udc_state() == 1){
					if (brightness_is_dimmed()){
						brightness_dim(0);
					}
					//draw_is = 1;
					set_cpu_freq(MAX_CPU_FREQ);
					udc_open_flag = 1;
					udc_draw_flag = 1;
					
					//system("/usr/bin/udc_connect.sh");
					printf("\n udc device connected! \n");
				}
			}

			//check change
			if (change_flag == 0){
				change_old_state = change_state;
				change_state = get_udc_state();
				if (get_usb_connect_state() == 1 &&
						change_state == POWER &&
						change_old_state == NORMAL){

						if (brightness_is_dimmed()){
							brightness_dim(0);
						}
						//draw_is = 1;
						brightness_write(10);
						change_flag = 1;
						//clear change pin
						set_change_state(1);
				}
			}
			udc_check_time = SDL_GetTicks() + UDC_DELAY_TIME;
		}


			update_display(screen, state);
		//SDL_SaveBMP(screen,"/usr/local/dmenu/test.bmp");
		//SDL_Delay(1000);
	} // end main loop
}

void draw_start_logo()
{
	char pic_path[256];
	int i;
	SDL_Surface* bootpic[BOOT_PIC_NUM];

	for(i = 0;i < BOOT_PIC_NUM;i++)
	{
		sprintf(pic_path,"%s%d.png",BOOT_PIC,i+1);
		bootpic[i] = IMG_Load(pic_path);
	}

	for(i = 0;i < BOOT_PIC_NUM;i++)
	{
		SDL_BlitSurface(bootpic[i],NULL,screen,NULL);
		SDL_Flip(screen);
		SDL_FreeSurface(bootpic[i]);
		bootpic[i] = NULL;
        if(get_tvdev_value() > 0)
            SDL_Delay(35);
        else
            SDL_Delay(50);
	}
}
int set_lcd_fg0(void)
{
  int ret, fbfd;

  if ((fbfd = open("/dev/fb0", O_RDWR)) == -1) {
      printf("++ ERR: can't open /dev/fb0 ++\n");
      return 0;
  }
  struct jz4750lcd_info jzlcd_info;
 
  ret = ioctl(fbfd, FBIO_GET_MODE, &jzlcd_info);
  __jzlcd_disable_fg1(jzlcd_info);
  __jzlcd_enable_fg0(jzlcd_info);

  if (__jzlcd_is_ipu_enable(jzlcd_info))
    __jzlcd_disable_ipu(jzlcd_info);

  if (__jzlcd_is_ipu_restart_enable(jzlcd_info))
    __jzlcd_disable_ipu_restart(jzlcd_info);

  ret = ioctl(fbfd, FBIODISPOFF, &jzlcd_info);
  ret = ioctl(fbfd, FBIO_DEEP_SET_MODE, &jzlcd_info);
  ret = ioctl(fbfd, FBIODISPON, &jzlcd_info);
  close(fbfd);

  return 0;
}
int main ( int argc, char** argv )
{
	printf("welcome go to dmenu!\n");
//	set_lcd_fg0();
    int i;
#if GAMELOCK_ENABLED
	if (lock_conf_load(NULL)) {
		printf("lock conf load fail!\n");
		return 1;
	}
#endif
	game_unlock_init();

	if(!version_info_conf_init()){
		printf("version conf load error!\n");
	}

	int rc = init();
	printf("Fully initialized\n");

#if 0
	if(access("/usr/etc/boot_pic_once",F_OK) == 0){
		draw_start_logo();
		system("rm /usr/etc/boot_pic_once -r");
		SDL_Delay(2000);
	}
#endif
	//loading_init(screen);
   	clear_a320_mode();
	start_flush_screen();

	if (rc==0) {
		listen(); //main loop
		deinit(SHUTDOWN);
		SDL_Quit(); 
		//alterac add
		if (sdl_to_fb_flag){
			//sdl_to_fb(0);
			//free_sdlface();
		}
		//end
		// all is well ;)
		log_debug("Exited cleanly");
	}
	return rc;
}
