#include "volume.h"
#include "resource.h"

extern cfg_t *cfg_main;

int volume_level;
int volume;
SDL_Surface* volume_status;
SDL_Rect volume_src_rect, volume_dst_rect;
static int mute_flag = 0;
int get_mute_flag()
{
		return mute_flag;
}

int volume_enabled()
{
    return cfg_getbool(cfg_main, "VolDisp");
}

void volume_init() 
{    
    
    volume_level  = (int)cfg_getint(cfg_main, "SndVol");
	if (volume_level < 3)
			mute_flag = 1;
	else
			mute_flag = 0;
	volume = volume_level/3;
	if (volume < 0)
			volume = 0;
	if (volume > 30)
			volume = 30;
#if 1
    volume_status = load_global_image("sound.png");
    
    //Initialize OSD positions
    init_rect_pos(&volume_dst_rect, 
        VOLUME_ICON_X, VOLUME_ICON_Y);
    
   // init_rect(&volume_src_rect, 0, 0,
     //   VOLUME_ICON_MIN_W, VOLUME_ICON_H);
     
#endif	
    //Set volume
    volume_set(volume_level);
}

void save_vol_to_file()
{
	char tmp_str[64];
	memset(tmp_str,0,64);
	sprintf(tmp_str,"sed -i '9 c\SndVol = %d'  /usr/etc/dmenu/dmenu.ini;sync",volume_level);
	system(tmp_str);
	SDL_Delay(50);
	add_crc_file(DMENU_CONF_FILE);
}
void volume_change(Direction dir) 
{
	mute_flag = 0;
	if (dir == PREV){
		volume --;
		if (volume < 0)
			volume = 0;
	}else{
		volume ++;
		if (volume > 30)
			volume = 30;
	}
	if (volume == 30)
		volume_level = 100;
	else if (volume == 0){
		audiosd_close();
		mute_flag = 1;
		volume_level = 0;
	}
	else
		volume_level = volume*3;
	printf("cur volume = %d\n",volume_level);
	volume_set(volume_level);

}
int get_cur_vol()
{
	return (volume_level/3);
}
static int old_level = 0;
void dmenu_set_vol(int level) 
{
#if SOUND_ENABLE
	int tmp_vol;
	if (level >= 30)
		tmp_vol = 100;
	else
		tmp_vol = level*3;
	if (tmp_vol <= 0){
		audiosd_close();
	}else{
		audiosd_open();
	}
	int volume_level1 = bound(tmp_vol, 0, 100);
	if (old_level != volume_level1){
		printf("dmenu set new  volume!\n");
		old_level = volume_level1;
		int oss_volume = volume_level1 | (volume_level1 << 8); // set volume for both channels
		int mixer = open(MIXER_DEVICE, O_WRONLY);
		if (ioctl(mixer, SOUND_MIXER_WRITE_VOLUME, &oss_volume) == -1) {
			printf("Failed opening mixer for write - VOLUME");
		}
		close(mixer);
	}
#endif
}


void volume_set(int level) 
{
#if SOUND_ENABLED
    int min = VOLUME_ICON_MIN_W, max = VOLUME_ICON_MAX_W;
    volume_level = bound(level, 0, 100);

    //Calculate clipping of volume rect
    volume_src_rect.w = min+(int)((max-min)*(volume_level/100.0f));

    int oss_volume = volume_level | (volume_level << 8); // set volume for both channels

	int mixer = open(MIXER_DEVICE, O_WRONLY);
	if (ioctl(mixer, SOUND_MIXER_WRITE_VOLUME, &oss_volume) == -1) {
        log_error("Failed opening mixer for write - VOLUME");
	}
	close(mixer);

    cfg_setint( cfg_main, "SndVol", (long)volume_level );
#endif
}

void volume_show(SDL_Surface* surface) 
{
	SDL_Surface* tmp_face;
	SDL_Color color = {255,255,255};
	char tmp_str[64];
	memset(tmp_str,0,64);
	sprintf(tmp_str,"%2d",volume);
	sdlDrawText(VOLUME_X,OSD_Y-3,tmp_str,color,surface);
    SDL_BlitSurface(volume_status, NULL, surface, &volume_dst_rect );    
	SDL_FreeSurface(tmp_face);
}

void volume_deinit() 
{
    log_debug("De-initializing");
    free_surface(volume_status);
}
